﻿//Kod kreskowy Pharmacode one-track
//(c) 2019 by Tomasz Lubinski
//www.algorytm.org

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace Barcodes.Pharmacode1
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Zainicjuj
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            pictureBox.Image = new Bitmap(pictureBox.Width, pictureBox.Height, PixelFormat.Format24bppRgb);
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));
            pictureBox.Refresh();
            whiteBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Sprawdza czy podany txt mozna zakodowac Pharmacode One-Track
        /// </summary>
        /// <param name="txt">tekst do zakodowania</param>
        /// <returns>true jezeli mozna zakodowac, false w przeciwnym razie</returns>
        public bool check(string txt)
        {
            int value;

            //sprawdz czy podano liczbe
            if (int.TryParse(txt, out value) == false)
                return false;

            //sprawdz czy podana liczba miesci sie w zakresie 3-131070
            if (value < 3 || value > 131070)
                return false;

            return true;
        }

        /// <summary>
        /// Generuj Pharmacode one-track
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Generate_Click(object sender, EventArgs e)
        {
            //Zainicjuj narzedzia do rysowania
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.SolidBrush blackBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            Font drawFont = new Font("Arial", 10);

            //Wyczysc obszar rysowania
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));

            //Sprawdz czy podany ciag mozna zakodowac
            string code = Code.Text;
            if (check(code) == false)
            {
                MessageBox.Show("Nieprawidłowa wartosc!\nMożna kodować liczby z zakresu 3 - 131070.", "Bład", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Wygeneruj i narysuj kod
            char[] bars = generateBars(code);
            int pos = 10;
            int width = 0;
            for (int i = 0; i < bars.Length; i++)
            {
                if (bars[i] == 'W')
                {
                    width = 6;
                }
                else
                {
                    width = 2;
                }
                //Narysuj pasek i przesun sie o jego szerokosc + dwukrotnosc paska waskiego
                graphics.FillRectangle(blackBrush, pos, 10, width, 50);
                pos += width + 4;
            }

            //Odswiez
            pictureBox.Refresh();

            //Zwolinij zasoby
            whiteBrush.Dispose();
            blackBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Generuje kod kreskowy
        /// </summary>
        /// <param name="code">string do zakodowania kodem kreskowym</param>
        /// <returns>zwraca szerokosc kolejnych paskow - 'W' pasek szeroki, 'N' pasek waski</returns>
        private char[] generateBars(string code)
        {
            int value;
            value = int.Parse(code);

            string bars = "";

            //Dodaj kolejne znaki
            while (value != 0)
            {
                if ((value % 2) == 0)
                {
                    bars = 'W' + bars;
                    value = (value - 2) / 2;
                }
                else
                {
                    bars = 'N' + bars;
                    value = (value - 1) / 2;
                }
            }

            return bars.ToArray();
        }
    }
}
